/**
 * @copyright (c) JTL-Software-GmbH
 * @license http://jtl-url.de/jtlshoplicense
 */
(function ($, document, window, viewport) {
    'use strict';

    var _stock_info = ['out-of-stock', 'in-short-supply', 'in-stock'],
        $v,
        ArticleClass = function () {
            this.init();
        };

    ArticleClass.DEFAULTS = {
        input: {
            id: 'a'
        },
        action: {
            compareList: 'Vergleichsliste',
            compareListRemove: 'Vergleichsliste.remove'
        },
        selector: {
            navBadgeUpdate: '#shop-nav li.compare-list-menu',
            navBadgeAppend: '#shop-nav li.cart-menu',
            boxContainer: 'section.box-compare'
        },
        modal: {
            id: 'modal-article-dialog',
            wrapper: '#result-wrapper',
            wrapper_modal: '#result-wrapper-modal'
        }
    };

    ArticleClass.prototype = {
        modalShown: false,
        modalView: null,

        constructor: ArticleClass,

        init: function () {
            this.options = ArticleClass.DEFAULTS;
            this.gallery = null;
        },

        onLoad: function () {
            if (this.isSingleArticle()) {
                var that = this;
                var form = $.evo.io().getFormValues('buy_form');

                if (typeof history.replaceState === 'function') {
                    history.replaceState({
                        a: form.a,
                        a2: form.VariKindArtikel || form.a,
                        url: document.location.href,
                        variations: {}
                    }, document.title, document.location.href);
                }

                window.addEventListener('popstate', function (event) {
                    if (event.state) {
                        that.setArticleContent(event.state.a, event.state.a2, event.state.url, event.state.variations);
                    }
                }, false);
            }
        },

        isSingleArticle: function () {
            return $('#buy_form').length > 0;
        },

        getWrapper: function (wrapper) {
            return typeof wrapper === 'undefined' ? $(this.options.modal.wrapper) : $(wrapper);
        },

        getCurrent: function ($item) {
            var $current = $item.hasClass('variation') ? $item : $item.closest('.variation');
            if ($current.context.tagName === 'SELECT') {
                $current = $item.find('option:selected');
            }

            return $current;
        },

        register: function (wrapper) {
            var $wrapper = this.getWrapper(wrapper);

            if (this.isSingleArticle()) {
                this.registerGallery();
                this.registerConfig();
                this.s360_registerSliders();
                this.s360_registerMisc();
            }

            this.registerSimpleVariations($wrapper);
            this.registerSwitchVariations($wrapper);
            this.registerImageSwitch($wrapper);
            this.registerArticleOverlay($wrapper);
            this.registerFinish($wrapper);
        },

        s360_registerSliders: function () {
            var updateThumbsIndicator =
                function (event, slick, currentSlide) {
                    var $nav = $('.s360-product-gallery-thumbs-nav');
                    if ($nav.length) {
                        var $nextThumb = $('.s360-product-gallery-thumbs-slider').find('.s360-product-gallery-thumb[data-slick-index="' + currentSlide + '"]');
                        if ($nextThumb.length) {
                            var width = $nextThumb.outerWidth(false);
                            var offsetX = $nextThumb.offset().left - $nextThumb.closest('.s360-product-gallery-thumbs-slider').offset().left;
                            $nav.find('.s360-product-gallery-thumbs-indicator').css({left: offsetX + "px", width: width + "px"});
                        }
                    }
                };
            $('.s360-product-gallery-images-slider:not(.slick-initialized)').slick().on('reInit afterChange', updateThumbsIndicator);
            $('.s360-product-gallery-thumbs-slider:not(.slick-initialized)').slick();

            updateThumbsIndicator(null, null, 0); // initialize thumbs indicator on first slide

            // register photoswipe
            var openPhotoSwipe = function () {
                var $images = $('.s360-product-gallery-images img');
                var photoswipeImageData = [];
                var currentIndex = 0;
                var $currentThumb = $('.s360-product-gallery-thumb.slick-current');
                if ($currentThumb.length) {
                    currentIndex = $currentThumb.data('slick-index');
                }
                var options = {
                    index: currentIndex,
                    history: false, // must be false, else closing PWSP triggers a reload of the variations
                    closeOnScroll: false,
                    closeOnVerticalDrag: false
                };
                $images.each(function () {
                    var $this = $(this);
                    var imageData = $this.data('list');
                    if (imageData) {
                        var image = {
                            src: imageData.lg.src,
                            w: imageData.lg.size.width,
                            h: imageData.lg.size.height
                        };
                        photoswipeImageData.push(image);
                    }
                });
                var $photoSwipeElement = $('.pswp');
                if ($photoSwipeElement.length) {
                    var photoswipeGallery = new PhotoSwipe($photoSwipeElement[0], PhotoSwipeUI_Default, photoswipeImageData, options);
                    photoswipeGallery.init();
                }
            };
            $('.s360-product-gallery-images').click(openPhotoSwipe);
        },

        s360_registerMisc: function () {
            $('body').trigger('article-variation-loaded.s360');
        },

        registerGallery: function () {
            var $gallery = $('#gallery');

            if ($gallery.length === 1) {
                this.gallery = $gallery.gallery();
                this.galleryIndex = 0;
                this.galleryLastIdent = '_';
            } else {
                this.gallery = null;
            }
        },

        registerConfig: function () {
            var that = this,
                config = $('.product-configuration')
                    .closest('form')
                    .find('input[type="radio"], input[type="checkbox"], input[type="number"], select');

            if (config.length > 0) {
                config.on('change', function () {
                    that.configurator();
                })
                    .keypress(function (e) {
                        if (e.which === 13) {
                            return false;
                        }
                    });
                that.configurator(true);
            }
        },

        registerSimpleVariations: function ($wrapper) {
            var that = this;
            $('.variations select', $wrapper).selectpicker({
                iconBase: 'fa',
                tickIcon: 'fa-check-square-o',
                hideDisabled: true,
                showTick: true,
                template: {
                    caret: '<span class="fa fa-angle-down"></span>'
                }

                /*mobile: true*/
            });

            $('.simple-variations input[type="radio"]', $wrapper)
                .on('change', function () {
                    var val = $(this).val(),
                        key = $(this).parent().data('key');
                    $('.simple-variations [data-key="' + key + '"]').removeClass('active');
                    $('.simple-variations [data-value="' + val + '"]').addClass('active');
                });

            $('.simple-variations input[type="radio"], .simple-variations select', $wrapper)
                .each(function (i, item) {
                    var $item = $(item),
                        wrapper = '#' + $item.closest('form').closest('div[id^="result-wrapper"]').attr('id');

                    /* old, undefined wrapper: # + $item.closest('form').closest('div').attr('id') */

                    $item.on('change', function () {
                        that.variationPrice($(this), true, wrapper);
                    });
                });
        },

        registerSwitchVariations: function ($wrapper) {
            var that = this;

            $('.switch-variations input[type="radio"], .switch-variations select', $wrapper)
                .each(function (i, item) {
                    var $item = $(item),
                        wrapper = '#' + $item.closest('form').closest('div[id^="result-wrapper"]').attr('id');

                    $item.on('change', function () {
                        that.variationSwitch($(this), false, wrapper);
                    });
                });

            // ie11 fallback
            if (typeof document.body.style.msTransform === 'string') {
                $('.variations label.variation', $wrapper)
                    .on('click', function (e) {
                        if (e.target.tagName === 'IMG') {
                            $(this).trigger('click');
                        }
                    });
            }
        },

        registerArticleOverlay: function ($wrapper) {
            var that = this;

            $('.quickview', $wrapper)
                .each(function (i, item) {
                    var $item = $(item),
                        formID = $item.data('target'),
                        wrapper = that.options.modal.wrapper_modal + '_' + formID,
                        srcWrapper = that.options.modal.wrapper + '_' + formID;

                    $item.on('click', function (event) {
                        event.preventDefault();
                        that.modalArticleDetail(this, wrapper, srcWrapper);
                    });
                });
            $wrapper.hover(null, function () {
                $(this).removeClass('active');
            })
        },

        registerImageSwitch: function ($wrapper) {
            var that = this,
                imgSwitch,
                gallery = this.gallery;

            imgSwitch = function (context, temporary, force) {

                var $context = $(context),
                    id = $context.attr('data-key'),
                    value = $context.attr('data-value'),
                    data = $context.data('list'),
                    title = $context.attr('data-title');
                if (!!data) {
                    if (typeof temporary === 'undefined') {
                        temporary = true;
                    }

                    var $gallery = $('.s360-product-gallery-images');
                    var singleImageMode = false;
                    if ($gallery.hasClass('no-thumbs')) {
                        singleImageMode = true;
                    }

                    // we need to add the image and re-initialize slick
                    var $imgsSlider = $('.s360-product-gallery-images-slider');

                    if(!$imgsSlider.length || !$imgsSlider.find('.s360-product-gallery-image').length) {
                        return;
                    }

                    $imgsSlider.slick('unslick');
                    // delete previously added images
                    if(temporary) {
                        $imgsSlider.find('.temporary-img').remove();
                    } else {
                        $imgsSlider.find('.virtual-img').remove();
                    }
                    // copy the first image element
                    var $img = $imgsSlider.find('.s360-product-gallery-image').first().clone();
                    // set our new values
                    var $imgTag = $img.find('img');
                    $imgTag.attr('src', data.lg.src);
                    $imgTag.data('list', data);
                    $imgTag.attr('title', title);
                    $img.addClass('virtual-img', true);
                    if(temporary) {
                        $img.addClass('temporary-img');
                    }
                    // set our new element as first element in the slider
                    $img.appendTo($imgsSlider);

                    // re-initialize slider
                    var lastSlideIndex = $imgsSlider.find('.s360-product-gallery-image').length - 1;
                    $imgsSlider.slick({initialSlide: lastSlideIndex});

                    // repeat for the thumbs slider
                    if(!singleImageMode && !temporary) {
                        var $thumbsSlider = $('.s360-product-gallery-thumbs-slider');
                        if(!$thumbsSlider.length || !$thumbsSlider.find('.s360-product-gallery-thumb').length) {
                            return;
                        }

                        $thumbsSlider.slick('unslick');
                        $thumbsSlider.find('.virtual-img').remove();
                        $img = $thumbsSlider.find('.s360-product-gallery-thumb').first().clone();
                        $imgTag = $img.find('img');
                        $imgTag.attr('src', data.md.src);
                        $imgTag.data('list', data);
                        $imgTag.attr('title', title);
                        $img.addClass('virtual-img', true);
                        $img.appendTo($thumbsSlider);
                        $thumbsSlider.slick();
                        // setting the thumb slider to initialize on the last slide does sadly not work, so we call the function bound to click events manually
                        var lastThumb = $thumbsSlider.find('.s360-product-gallery-thumb').last();
                        $thumbsSlider.slick('selectHandler', jQuery.Event('s360-custom-event', {target: lastThumb}));
                    } else if(!singleImageMode) {
                        // re-initialize thumb slider so it shows the first element
                        // in this case, also be sure to re-initialize the thumb slider
                        var $thumbsSlider = $('.s360-product-gallery-thumbs-slider');
                        if($thumbsSlider.length && $thumbsSlider.find('.s360-product-gallery-thumb').length) {
                            $thumbsSlider.slick('unslick');
                            $thumbsSlider.slick();
                        }
                    }
                }
            };

            var imgSwitchReset = function() {
                var $imgsSlider = $('.s360-product-gallery-images-slider');

                if($imgsSlider.length && $imgsSlider.find('.temporary-img').length) {
                    $imgsSlider.slick('unslick');
                    $imgsSlider.find('.temporary-img').remove();
                    $imgsSlider.slick();

                    // in this case, also be sure to re-initialize the thumb slider
                    var $thumbsSlider = $('.s360-product-gallery-thumbs-slider');
                    if($thumbsSlider.length && $thumbsSlider.find('.s360-product-gallery-thumb').length) {
                        $thumbsSlider.slick('unslick');
                        $thumbsSlider.slick();
                    }
                }
            };

            $('.variations .bootstrap-select select', $wrapper)
                .change(function () {
                    var sel = $(this).find('[value=' + this.value + ']'),
                        cont = $(this).closest('.variations');

                    if (cont.hasClass('simple-variations')) {
                        imgSwitch(sel, false, false);
                    } else {
                        imgSwitch(sel, true, false);
                    }
                });

            if (!isTouchCapable() || ResponsiveBootstrapToolkit.current() !== 'xs') {
                $('.variations .bootstrap-select .dropdown-menu li', $wrapper)
                    .hover(function () {
                        var tmp_idx = parseInt($(this).attr('data-original-index')) + 1,
                            rule = 'select option:nth-child(' + tmp_idx + ')',
                            sel = $(this).closest('.bootstrap-select').find(rule);
                        imgSwitch(sel);
                    }, function () {
                        imgSwitchReset();
                    });
            }

            $('.variations.simple-variations .variation', $wrapper)
                .click(function () {
                    imgSwitch(this, false);
                });

            if (!isTouchCapable() || ResponsiveBootstrapToolkit.current() !== 'xs') {
                $('.variations .variation', $wrapper)
                    .hover(function () {
                        imgSwitch(this);
                    }, function () {
                        imgSwitchReset();
                    });
            }
        },

        registerFinish: function ($wrapper) {
            $('#jump-to-votes-tab', $wrapper).click(function () {
                $('#content a[href="#tab-votes"]').tab('show');
            });

            if (!this.isSingleArticle()) {
                var that = this;

                $('.product-cell.basket-enabled')
                    .on('click', function (event) {
                        if (isTouchCapable() && ResponsiveBootstrapToolkit.current() !== 'xs') {
                            var $this = $(this);

                            if (!$this.hasClass('active')) {
                                event.preventDefault();
                                event.stopPropagation();
                                $('.product-cell').removeClass('active');
                                $this.addClass('active');
                            }
                        }
                    });
            }

            this.registerProductActions($wrapper);
        },

        registerProductActions: function ($wrapper) {
            var that = this;

            $('*[data-toggle="product-actions"] button', $wrapper)
                .on('click', function (event) {
                    var data = $(this.form).serializeObject();

                    if (that.handleProductAction(this, data)) {
                        event.preventDefault();
                    }
                });
            $('a[data-toggle="product-actions"]', $wrapper)
                .on('click', function (event) {
                    var data = $(this).data('value');
                    this.name = $(this).data('name');

                    if (that.handleProductAction(this, data)) {
                        event.preventDefault();
                    }
                });
        },

        loadModalArticle: function (url, wrapper, done, fail) {
            var that = this,
                $wrapper = this.getWrapper(wrapper),
                id = wrapper.substring(1),
                $modalBody = $('.modal-body', this.modalView);

            $wrapper.addClass('loading');

            $.ajax(url, {data: {'isAjax': 1, 'quickView': 1}})
                .done(function (data) {
                    var $html = $('<div />').html(data);
                    var $headerCSS = $html.find('link[type="text/css"]');
                    var $headerJS = $html.find('head script[src][src!=""]');
                    var $bodyJS = $html.find('body script[src][src!=""]');
                    var content = $html.find(that.options.modal.wrapper).html();

                    $headerCSS.each(function (pos, item) {
                        var $cssLink = $('head link[href="' + item.href + '"]');
                        if ($cssLink.length === 0) {
                            $('head').append('<link rel="stylesheet" type="text/css" href="' + item.href + '" >');
                        }
                    });

                    $headerJS.each(function (pos, item) {
                        if (typeof item.src !== 'undefined' && item.src.length > 0) {
                            var $jsLink = $('head script[src="' + item.src + '"]');
                            if ($jsLink.length === 0) {
                                $('head').append('<script type="text/javascript" src="' + item.src + '" >');
                            }
                        }
                    });

                    $bodyJS.each(function (pos, item) {
                        if (typeof item.src !== 'undefined' && item.src.length > 0) {
                            var $jsLink = $('body script[src="' + item.src + '"]');
                            if ($jsLink.length === 0) {
                                $('body').append('<script type="text/javascript" src="' + item.src + '" >');
                            }
                        }
                    });

                    $modalBody.html($('<div id="' + id + '" />').html(content));

                    var $modal = $modalBody.closest(".modal-dialog"),
                        title = $modal.find('.modal-body h1'),
                        $config = $('#product-configurator', $modalBody);

                    if ($config.length > 0) {
                        // Configurator in child article!? Currently not supported!
                        $config.remove();
                        $modalBody.addClass('loading');
                        var spinner = $.evo.spinner($modalBody.get(0));
                        location.href = url;
                    }

                    if (title.length > 0 && title.text().length > 0) {
                        $modal.find('.modal-title').text(title.text());
                        title.remove();
                    }

                    $('form', $modalBody).on('submit', function (event) {
                        event.preventDefault();

                        var $form = $(this);
                        var data = $form.serializeObject();
                        if (data['VariKindArtikel']) {
                            data['a'] = data['VariKindArtikel'];
                        }

                        $.evo.basket().addToBasket($form, data);
                        that.modalView.modal('hide');
                    });

                    if (typeof done === 'function') {
                        done();
                    }
                })
                .fail(function () {
                    if (typeof fail === 'function') {
                        fail();
                    }
                })
                .always(function () {
                    $wrapper.removeClass('loading');
                });
        },

        addToComparelist: function (data) {
            var productId = parseInt(data[this.options.input.id]);
            if (productId > 0) {
                var that = this;
                $.evo.io().call('pushToComparelist', [productId], that, function (error, data) {
                    if (error) {
                        return;
                    }

                    var response = data.response;

                    if (response) {
                        switch (response.nType) {
                            case 0: // error
                                var errorlist = '<ul><li>' + response.cHints.join('</li><li>') + '</li></ul>';
                                eModal.alert({
                                    title: response.cTitle,
                                    message: errorlist,
                                    keyboard: true,
                                    tabindex: -1
                                });
                                break;
                            case 1: // forwarding
                                window.location.href = response.cLocation;
                                break;
                            case 2: // added to comparelist
                                that.updateComparelist(response);
                                eModal.alert({
                                    title: response.cTitle,
                                    message: response.cNotification,
                                    keyboard: true,
                                    tabindex: -1
                                });
                                break;
                        }
                    }
                });

                return true;
            }

            return false;
        },

        removeFromCompareList: function (data) {
            var productId = parseInt(data[this.options.input.id]);
            if (productId > 0) {
                var that = this;
                $.evo.io().call('removeFromComparelist', [productId], that, function (error, data) {
                    if (error) {
                        return;
                    }

                    var response = data.response;

                    if (response) {
                        switch (response.nType) {
                            case 0: // error
                                var errorlist = '<ul><li>' + response.cHints.join('</li><li>') + '</li></ul>';
                                eModal.alert({
                                    title: response.cTitle,
                                    message: errorlist,
                                    keyboard: true,
                                    tabindex: -1
                                });
                                break;
                            case 1: // forwarding
                                window.location.href = response.cLocation;
                                break;
                            case 2: // removed from comparelist
                                that.updateComparelist(response);
                                break;
                        }
                    }
                });

                return true;
            }

            return false;
        },

        updateComparelist: function (data) {
            var $badgeUpd = $(this.options.selector.navBadgeUpdate);

            if (data.nCount > 1 && data.cNavBadge.length) {
                var badge = $(data.cNavBadge);
                if ($badgeUpd.size() > 0) {
                    $badgeUpd.replaceWith(badge);
                } else {
                    $(this.options.selector.navBadgeAppend).before(badge);
                }

                badge.on('click', '.popup', function (e) {
                    var url = e.currentTarget.href;
                    url += (url.indexOf('?') === -1) ? '?isAjax=true' : '&isAjax=true';
                    eModal.ajax({
                        size: 'lg',
                        url: url,
                        keyboard: true,
                        tabindex: -1
                    });
                    e.stopPropagation();

                    return false;
                });
            } else if ($badgeUpd.size() > 0) {
                $badgeUpd.remove();
            }

            var $list = $(this.options.selector.boxContainer);

            if ($list.size() > 0) {
                if (data.cBoxContainer.length) {
                    var $boxContent = $(data.cBoxContainer);
                    this.registerProductActions($boxContent);
                    $list.replaceWith($boxContent).removeClass('hidden');
                } else {
                    $list.html('').addClass('hidden');
                }
            }
        },

        handleProductAction: function (action, data) {
            switch (action.name) {
                case this.options.action.compareList:
                    return this.addToComparelist(data);
                case this.options.action.compareListRemove:
                    return this.removeFromCompareList(data);
            }

            return false;
        },

        configurator: function (init) {
            if (this.isSingleArticle()) {
                var that = this,
                    container = $('#cfg-container'),
                    sidebar = $('#product-configuration-sidebar'),
                    width,
                    form;

                if (container.length === 0) {
                    return;
                }

                if (viewport.current() !== 'lg') {
                    sidebar.removeClass('affix');
                }

                if (!sidebar.hasClass('affix')) {
                    sidebar.css('width', '');
                }

                sidebar.css('width', sidebar.width());


                if (init) {
                    sidebar.affix({
                        offset: {
                            top: function () {
                                var top = container.offset().top - $('#evo-main-nav-wrapper.affix').outerHeight(true);
                                if (viewport.current() !== 'lg') {
                                    top = 999999;
                                }
                                return top;
                            },
                            bottom: function () {
                                var bottom = $('body').height() - (container.height() + container.offset().top);
                                if (viewport.current() !== 'lg') {
                                    bottom = 999999;
                                }
                                return bottom;
                            }
                        }
                    });

                    container.find('select').selectpicker({
                        iconBase: 'fa',
                        tickIcon: 'fa-check-square-o',
                        hideDisabled: true,
                        showTick: true,
                        template: {
                            caret: '<span class="fa fa-angle-down"></span>'
                        }
                    });
                }

                $('#buy_form').find('*[data-selected="true"]')
                    .attr('checked', true)
                    .attr('selected', true)
                    .attr('data-selected', null);

                form = $.evo.io().getFormValues('buy_form');

                $.evo.io().call('buildConfiguration', [form], that, function (error, data) {
                    var result,
                        i,
                        j,
                        item,
                        cBeschreibung,
                        quantityWrapper,
                        grp,
                        value,
                        enableQuantity,
                        nNetto,
                        quantityInput;
                    if (error) {
                        $.evo.error(data);
                        return;
                    }
                    result = data.response;

                    if (!result.oKonfig_arr) {
                        $.evo.error('Missing configuration groups');
                        return;
                    }

                    // global price
                    nNetto = result.nNettoPreise;
                    that.setPrice(result.fGesamtpreis[nNetto], result.cPreisLocalized[nNetto], result.cPreisString);
                    that.setStockInformation(result.cEstimatedDelivery);

                    $('#content .summary').html(result.cTemplate);

                    sidebar.affix('checkPosition');

                    // groups
                    for (i = 0; i < result.oKonfig_arr.length; i++) {
                        grp = result.oKonfig_arr[i];
                        quantityWrapper = that.getConfigGroupQuantity(grp.kKonfiggruppe);
                        quantityInput = that.getConfigGroupQuantityInput(grp.kKonfiggruppe);
                        if (grp.bAktiv) {
                            enableQuantity = grp.bAnzahl;
                            for (j = 0; j < grp.oItem_arr.length; j++) {
                                item = grp.oItem_arr[j];
                                if (item.bAktiv) {
                                    if (item.cBildPfad) {
                                        that.setConfigItemImage(grp.kKonfiggruppe, item.cBildPfad.cPfadKlein);
                                    } else {
                                        that.setConfigItemImage(grp.kKonfiggruppe, grp.cBildPfad);
                                    }
                                    that.setConfigItemDescription(grp.kKonfiggruppe, item.cBeschreibung);
                                    enableQuantity = item.bAnzahl;
                                    if (!enableQuantity) {
                                        quantityInput
                                            .attr('min', item.fInitial)
                                            .attr('max', item.fInitial)
                                            .val(item.fInitial)
                                            .attr('disabled', true);
                                        if (item.fInitial === 1) {
                                            quantityWrapper.slideUp(200);
                                        } else {
                                            quantityWrapper.slideDown(200);
                                        }
                                    } else {
                                        if (quantityWrapper.css('display') === 'none' && !init) {
                                            quantityInput.val(item.fInitial);
                                        }
                                        quantityWrapper.slideDown(200);
                                        quantityInput
                                            .attr('disabled', false)
                                            .attr('min', item.fMin)
                                            .attr('max', item.fMax);
                                        value = quantityInput.val();
                                        if (value < item.fMin || value > item.fMax) {
                                            quantityInput.val(item.fInitial);
                                        }
                                    }
                                }
                            }
                        }
                        else {
                            that.setConfigItemDescription(grp.kKonfiggruppe, '');
                            quantityInput.attr('disabled', true);
                            quantityWrapper.slideUp(200);
                        }
                    }

                    $.evo.trigger('priceChanged', result);
                    container.find('select').selectpicker('refresh');
                });
            }
        },

        variationRefreshAll: function ($wrapper) {
            $('.variations select', $wrapper).selectpicker('refresh');
        },

        getConfigGroupQuantity: function (groupId) {
            return $('.cfg-group[data-id="' + groupId + '"] .quantity');
        },

        getConfigGroupQuantityInput: function (groupId) {
            return $('.cfg-group[data-id="' + groupId + '"] .quantity input');
        },

        getConfigGroupImage: function (groupId) {
            return $('.cfg-group[data-id="' + groupId + '"] .group-image img');
        },

        setConfigItemImage: function (groupId, img) {
            $('.cfg-group[data-id="' + groupId + '"] .group-image img').attr('src', img).first();
        },

        setConfigItemDescription: function (groupId, itemBeschreibung) {
            var groupItems = $('.cfg-group[data-id="' + groupId + '"] .group-items');
            var descriptionDropdownContent = groupItems.find('#filter-collapsible_dropdown_' + groupId + '');
            var descriptionDropdownContentHidden = groupItems.find('.hidden');
            var descriptionCheckdioContent = groupItems.find('div[id^="filter-collapsible_checkdio"]');
            var multiselect = groupItems.find('select').attr("multiple");

            //  Bisher kein Content mit einer Beschreibung vorhanden, aber ein Artikel mit Beschreibung ausgewÃ¤hlt
            if (descriptionDropdownContentHidden.length > 0 && descriptionCheckdioContent.length === 0 && itemBeschreibung.length > 0 && multiselect !== "multiple") {
                groupItems.find('a[href="#filter-collapsible_dropdown_' + groupId + '"]').removeClass('hidden');
                descriptionDropdownContent.replaceWith('<div id="filter-collapsible_dropdown_' + groupId + '" class="collapse top10 panel-body">' + itemBeschreibung + '</div>');
                //  Bisher Content mit einer Beschreibung vorhanden, aber ein Artikel ohne Beschreibung ausgewÃ¤hlt
            } else if (descriptionDropdownContentHidden.length === 0 && descriptionCheckdioContent.length === 0 && itemBeschreibung.length === 0 && multiselect !== "multiple") {
                groupItems.find('a[href="#filter-collapsible_dropdown_' + groupId + '"]').addClass('hidden');
                descriptionDropdownContent.addClass('hidden');
                //  Bisher Content mit einer Beschreibung vorhanden und ein Artikel mit Beschreibung ausgewÃ¤hlt
            } else if (descriptionDropdownContentHidden.length === 0 && descriptionCheckdioContent.length === 0 && itemBeschreibung.length > 0 && multiselect !== "multiple") {
                descriptionDropdownContent.replaceWith('<div id="filter-collapsible_dropdown_' + groupId + '" class="collapse top10 panel-body">' + itemBeschreibung + '</div>');
            }
        },

        setPrice: function (price, fmtPrice, priceLabel, wrapper) {
            var $wrapper = this.getWrapper(wrapper);

            if (this.isSingleArticle()) {
                $('#product-offer .price, .updatable-price .price', $wrapper).html(fmtPrice);
                if (priceLabel.length > 0) {
                    $('#product-offer .price_label, .updatable-price .price_label', $wrapper).html(priceLabel);
                }
            } else {
                var $price = $('.price_wrapper', $wrapper);

                $('.price span:first-child', $price).html(fmtPrice);
                if (priceLabel.length > 0) {
                    $('.price_label', $price).html(priceLabel);
                }
            }

            $.evo.trigger('changed.article.price', {price: price});
        },

        setStockInformation: function (cEstimatedDelivery, wrapper) {
            if(typeof cEstimatedDelivery === "undefined") {
                // prevent setting undefined values, e.g. when JTL Shop 4.05 does not return a valid value
                return;
            }
            var $wrapper = this.getWrapper(wrapper);

            $('.delivery-status .estimated-delivery span', $wrapper).html('(' + cEstimatedDelivery + ')');
        },

        setStaffelPrice: function (prices, fmtPrices, wrapper) {
            var $wrapper = this.getWrapper(wrapper),
                $container = $('#product-offer', $wrapper);

            $.each(fmtPrices, function (index, value) {
                $('.bulk-price-' + index + ' .bulk-price', $container).html(value);
            });
        },

        setVPEPrice: function (fmtVPEPrice, VPEPrices, fmtVPEPrices, wrapper) {
            var $wrapper = this.getWrapper(wrapper),
                $container = $('#product-offer', $wrapper);

            $('.base-price .value', $container).html(fmtVPEPrice);
            $.each(fmtVPEPrices, function (index, value) {
                $('.bulk-price-' + index + ' .bulk-base-price', $container).html(value);
            });
        },

        /**
         * @deprecated since 4.05 - use setArticleWeight instead
         */
        setUnitWeight: function (UnitWeight, newUnitWeight) {
            $('#article-tabs .product-attributes .weight-unit').html(newUnitWeight);
        },

        setArticleWeight: function (ArticleWeight, wrapper) {
            if (this.isSingleArticle()) {
                var $articleTabs = $('#article-tabs');

                if ($.isArray(ArticleWeight)) {
                    $('.product-attributes .weight-unit', $articleTabs).html(ArticleWeight[0][1]);
                    $('.product-attributes .weight-unit-article', $articleTabs).html(ArticleWeight[1][1]);
                } else {
                    $('.product-attributes .weight-unit', $articleTabs).html(ArticleWeight);
                }
            } else {
                var $wrapper = this.getWrapper(wrapper);

                if ($.isArray(ArticleWeight)) {
                    $('.attr-weight .value', $wrapper).html(ArticleWeight[0][1]);
                    $('.attr-weight.weight-unit-article .value', $wrapper).html(ArticleWeight[1][1]);
                } else {
                    $('.attr-weight .value', $wrapper).html(ArticleWeight);
                }
            }

        },

        setProductNumber: function (productNumber, wrapper) {
            var $wrapper = this.getWrapper(wrapper);

            $('#product-offer span[itemprop="sku"]', $wrapper).html(productNumber);
        },

        setArticleContent: function (id, variation, url, variations, wrapper) {
            var $wrapper = this.getWrapper(wrapper),
                listStyle = $('#product-list').length > 0 && $('#product-list').data('list-style') ? $('#product-list').data('list-style') : 'gallery',
                $spinner = $.evo.spinner($wrapper.get(0));

            if (this.modalShown) {
                // modal box article loading logic
                this.loadModalArticle(url, wrapper,
                    function () {
                        var article = new ArticleClass();
                        article.register(wrapper);
                        $spinner.stop();
                    },
                    function () {
                        $spinner.stop();
                        $.evo.error('Error loading ' + url);
                    }
                );
            } else if (this.isSingleArticle()) {
                // singe article detail page loading logic
                $.evo.loadContent(url, function (content) {
                    $.evo.register();
                    $.evo.article().register(wrapper);

                    $(variations).each(function (i, item) {
                        $.evo.article().variationSetVal(item.key, item.value, wrapper);
                    });

                    if (document.location.href !== url) {
                        history.pushState({a: id, a2: variation, url: url, variations: variations}, "", url);
                    }

                    $spinner.stop();
                }, function () {
                    $.evo.error('Error loading ' + url);
                    $spinner.stop();
                }, false, wrapper);
            } else {
                // list loading logic
                var basketParam = $wrapper.data('basket');
                var variationParam = $wrapper.data('variation');
                var maxVariationCountParam = $wrapper.data('max-variation');
                var maxWerteCountParam = $wrapper.data('max-variation-value');
                var fullWidthParam = $wrapper.data('fullwidth');
                var quickviewEnabledParam = $wrapper.data('quickview-enabled');
                var finalUrl = url + (url.indexOf('?') >= 0 ? '&' : '?') + 'isListStyle=' + listStyle;
                if(basketParam) {
                    finalUrl = finalUrl + '&basket='+basketParam;
                }
                if(variationParam) {
                    finalUrl = finalUrl + '&variation='+variationParam;
                }
                if(maxVariationCountParam) {
                    finalUrl = finalUrl + '&maxVariationCount='+maxVariationCountParam;
                }
                if(maxWerteCountParam) {
                    finalUrl = finalUrl + '&maxWerteCount='+maxWerteCountParam;
                }
                if(fullWidthParam) {
                    finalUrl = finalUrl + '&fullwidth='+maxWerteCountParam;
                }
                if(quickviewEnabledParam) {
                    finalUrl = finalUrl + '&quickviewEnabled='+quickviewEnabledParam;
                }
                $.evo.loadContent(finalUrl, function (content) {
                    $.evo.imagebox(wrapper);
                    $.evo.article().register(wrapper);

                    $('[data-toggle="basket-add"]', $(wrapper)).on('submit', function (event) {
                        event.preventDefault();
                        event.stopPropagation();

                        var $form = $(this);
                        var data = $form.serializeObject();
                        data['a'] = variation;

                        $.evo.basket().addToBasket($form, data);
                    });

                    $(variations).each(function (i, item) {
                        $.evo.article().variationSetVal(item.key, item.value, wrapper);
                    });

                    if (!$wrapper.hasClass('basket-enabled')) {
                        $.evo.autoheight();
                    }
                    $spinner.stop();
                }, function () {
                    $.evo.error('Error loading ' + url);
                    $spinner.stop();
                }, false, wrapper);
            }
        },

        variationResetAll: function (wrapper) {
            var $wrapper = this.getWrapper(wrapper);

            $('.variation[data-value] input:checked', $wrapper).prop('checked', false);
            $('.variations select option', $wrapper).prop('selected', false);
            $('.variations select', $wrapper).selectpicker('refresh');
        },

        variationDisableAll: function (wrapper) {
            var $wrapper = this.getWrapper(wrapper);

            $('.swatches-selected', $wrapper).text('');
            $('[data-value].variation', $wrapper).each(function (i, item) {
                $(item)
                    .removeClass('active')
                    .removeClass('loading')
                    .addClass('not-available');
                $.evo.article()
                    .removeStockInfo($(item));
            });
        },

        variationSetVal: function (key, value, wrapper) {
            var $wrapper = this.getWrapper(wrapper);

            $('[data-key="' + key + '"]', $wrapper).val(value);
        },

        variationEnable: function (key, value, wrapper) {
            var $wrapper = this.getWrapper(wrapper),
                $item = $('[data-value="' + value + '"].variation', $wrapper);

            $item.removeClass('not-available');
        },

        variationActive: function (key, value, def, wrapper) {
            var $wrapper = this.getWrapper(wrapper),
                $item = $('[data-value="' + value + '"].variation', $wrapper);

            $item.addClass('active')
                .removeClass('loading')
                .find('input')
                .prop('checked', true)
                .end()
                .prop('selected', true);

            $('[data-id="' + key + '"].swatches-selected')
                .text($item.attr('data-original'));
        },

        removeStockInfo: function($item) {
            var type = $item.attr('data-type'),
                elem,
                label,
                wrapper;

            switch (type) {
                case 'option':
                    label = $item.data('content');
                    wrapper = $('<div />').append(label);
                    $(wrapper)
                        .find('.label-not-available')
                        .remove();
                    label = $(wrapper).html();
                    $item.data('content', label)
                        .attr('data-content', label);
                    break;
                case 'radio':
                    elem = $item.find('.label-not-available');
                    if (elem.length === 1) {
                        $(elem).remove();
                    }
                    break;
                case 'swatch':
                    if ($item.data('bs.tooltip')) {
                        $item.tooltip('destroy');
                        $item.attr('title', $item.attr('data-title'));
                    }
                    break;
            }

            $item.removeAttr('data-stock');
        },

        variationInfo: function(value, status, note, wrapper) {
            // wrapper may or may not be given, depending on the underlying shop version
            // the input to this function comes indirectly via io_inc.php
            var $item;
            if(typeof wrapper !== 'undefined') {
                var $itemWrapper = this.getWrapper(wrapper);
                $item = $('[data-value="' + value + '"].variation', $itemWrapper);
            } else {
                $item = $('[data-value="' + value + '"].variation');
            }
            var type = $item.attr('data-type');
            var text;
            var content;
            var label;
            var $wrapper; // not to be confused with the itemWrapper

            $item.attr('data-stock', _stock_info[status]);

            switch (type) {
                case 'option':
                    text     = ' (' + note + ')';
                    content  = $item.data('content');
                    $wrapper = $('<div />');

                    $wrapper.append(content);
                    $wrapper
                        .find('.label-not-available')
                        .remove();

                    label = $('<span />')
                        .addClass('label label-default label-not-available')
                        .text(note);

                    $wrapper.append(label);

                    $item.data('content', $wrapper.html())
                        .attr('data-content', $wrapper.html());

                    $item.closest('select')
                        .selectpicker('refresh');
                    break;
                case 'radio':
                    $item.find('.label-not-available')
                        .remove();

                    label = $('<span />')
                        .addClass('label label-default label-not-available')
                        .text(note);

                    $item.append(label);
                    break;
                case 'swatch':
                    $item.attr('title', note);
                    window.setTimeout(function () {
                        $item.tooltip({
                            title: note,
                            trigger: 'hover',
                            container: 'body'
                        });
                    }, 300);
                    break;
            }
        },

        variationSwitch: function ($item, animation, wrapper) {
            if ($item) {
                var formID = $item.closest('form').attr('id'),
                    $current = this.getCurrent($item),
                    key = $current.data('key'),
                    value = $current.data('value'),
                    io = $.evo.io(),
                    args = io.getFormValues(formID),
                    $spinner = null,
                    $wrapper = this.getWrapper(wrapper);

                if (animation) {
                    $wrapper.addClass('loading');
                    $spinner = $.evo.spinner();
                } else {
                    $('.updatingStockInfo', $wrapper).show();
                }

                $current.addClass('loading');
                args.wrapper = wrapper;

                $.evo.article()
                    .variationDispose(wrapper);

                io.call('checkVarkombiDependencies', [args, key, value], $item, function (error, data) {
                    $wrapper.removeClass('loading');
                    if (animation) {
                        $spinner.stop();
                    }
                    $('.updatingStockInfo', $wrapper).hide();
                    if (error) {
                        $.evo.error('checkVarkombiDependencies');
                    }
                });
            }
        },

        variationPrice: function ($item, animation, wrapper) {
            var formID = $item.closest('form').attr('id'),
                $wrapper = this.getWrapper(wrapper),
                io = $.evo.io(),
                args = io.getFormValues(formID),
                $spinner = null;

            if (animation) {
                $wrapper.addClass('loading');
                $spinner = $.evo.spinner();
            }

            args.wrapper = wrapper;

            io.call('checkDependencies', [args], null, function (error, data) {
                $wrapper.removeClass('loading');
                if (animation) {
                    $spinner.stop();
                }
                if (error) {
                    $.evo.error('checkDependencies');
                }
            });
        },

        modalArticleDetail: function (item, wrapper, srcWrapper) {
            var that = this,
                title = $(srcWrapper).find('h4.title').text(),
                image = $(srcWrapper).find('.image-content').html(),
                url = $(item).data('src');

            if (typeof this.modalView === 'undefined' || this.modalView === null) {
                this.modalView = $(
                    '<div id="' + this.options.modal.id + '" class="modal fade" role="dialog" tabindex="-1" >' +
                    '   <div class="modal-dialog modal-lg">' +
                    '       <div class="modal-content">' +
                    '           <div class="modal-header">' +
                    '               <button type="button" class="x close" data-dismiss="modal">&times;</button>' +
                    '               <h4 class="modal-title">' + title + '</h4>' +
                    '           </div>' +
                    '           <div class="modal-body"><div id="' + wrapper.substring(1) + '" style="min-height:100px">' + image + '</div></div>' +
                    '       </div>' +
                    '   </div>' +
                    '</div>');
                this.modalView
                    .on('hidden.bs.modal', function () {
                        $('.modal-body', that.modalView).html('<div id="' + wrapper.substring(1) + '" style="min-height:100px" />');
                        $('.modal-title', that.modalView).html('');
                        that.modalView
                            .off('shown.bs.modal');
                        that.modalShown = false;
                    });
            } else {
                $('.modal-title', that.modalView).html(title);
                $('.modal-body', that.modalView).html('<div id="' + wrapper.substring(1) + '" style="min-height:100px">' + image + '</div>');
            }

            this.modalView
                .on('shown.bs.modal', function () {
                    var $spinner = $.evo.spinner($(wrapper).get(0));

                    that.modalShown = true;
                    that.loadModalArticle(url, wrapper,
                        function () {
                            var article = new ArticleClass();
                            article.register(wrapper);
                            $spinner.stop();
                        },
                        function () {
                            $spinner.stop();
                            $.evo.error('Error loading ' + params.url);
                        }
                    );
                })
                .modal('show');
        },

        variationDispose: function (wrapper) {
            var $wrapper = this.getWrapper(wrapper);
            $('[role="tooltip"]', $wrapper).remove();
        }
    };

    $v = new ArticleClass();
    var ie = /(msie|trident)/i.test(navigator.userAgent) ? navigator.userAgent.match(/(msie |rv:)(\d+(.\d+)?)/i)[2] : false;
    if (ie && parseInt(ie) <= 9) {
        $(document).ready(function () {
            $v.onLoad();
            $v.register();
        });
    } else {
        $(window).on('load', function () {
            $v.onLoad();
            $v.register();
        });
    }

    $(window).resize(
        viewport.changed(function () {
            $v.configurator();
        })
    );

    // PLUGIN DEFINITION
    // =================
    $.evo.article = function () {
        return $v;
    };
})(jQuery, document, window, ResponsiveBootstrapToolkit);