/**
 * @copyright (c) Solution 360 GmbH
 * Handles automatic positioning of elements in the targeted row.
 *
 * Needs bootstrap cols as direct children of the target container!
 * DOES NOT ADJUST TO SIZE CHANGES in order to be a bit less hard on the browser resources.
 */
(function ($) {

    function s360_autorow() {
        // position boxes that can be inserted into rows dynamically, for now we assume that items in rows all are either equal column size or full-width (-12)
        $('[data-target-row]').each(function (idx, elem) {
            var $this = $(elem);
            var targetRow = $this.data('target-row');
            var $targetContainer = $($this.data('target-container'));
            var viewport = $('#vp-indicator > div:visible').last().data('vp');
            var testViewports = null;
            switch(viewport) {
                case 'lg':
                    testViewports = ['lg', 'md', 'sm', 'xs'];
                    break;
                case 'md':
                    testViewports = ['md', 'sm', 'xs'];
                    break;
                case 'sm':
                    testViewports = ['sm', 'xs'];
                    break;
                case 'xs':
                    testViewports = ['xs'];
                    break;
            }

            // let's step through targetContainer
            var itemPositioned = false;
            var currentRow = 1;
            var currentRowSum = 0;

            $targetContainer.children('[class*="col-"]').each(function(idx, elem) {
                var $col = $(elem);
                if(currentRow === targetRow && currentRowSum === 0 || targetRow < 1) {
                    $col.before($this);
                    itemPositioned = true;
                }

                var classFound = false;
                // now iterate through the *valid* breakpoints and through the columns counts to find the relevant column size.
                for(var i = 0; i < testViewports.length; i++) {
                    for(var j = 1; j <= 12; j++) {
                        if($col.hasClass('col-'+testViewports[i]+'-'+j)) {
                            currentRowSum = currentRowSum + j;
                            classFound = true;
                            break;
                        }
                    }
                    if(classFound) {
                        break;
                    }
                }
                if(currentRowSum >= 12) {
                    currentRowSum = 0;
                    currentRow++;
                }
            });

            if(!itemPositioned) {
                // item was not positioned after stepping through all elements. Just add it to the end.
                $targetContainer.append($this);
            }
        });
    }

    // CALLS
    // ===============
    $(document).ready(function () {
        s360_autorow();
    });
})(jQuery);
